@@ -17,7 +17,9 @@ import logging
 import random
 import sys
 import time
+import warnings
 
+from openstack import connection
 import six
 
 from metalsmith import _config
@@ -26,6 +28,7 @@ from metalsmith import _os_api
 from metalsmith import _scheduler
 from metalsmith import _utils
 from metalsmith import exceptions
+from metalsmith import sources
 
 
 LOG = logging.getLogger(__name__)
@@ -43,14 +46,30 @@ class Provisioner(object):
         to use when making API requests. Mutually exclusive with **session**.
     :param dry_run: boolean value, set to ``True`` to prevent any API calls
         from being actually made.
+
+    :ivar connection: `openstacksdk` `Connection` object used for accessing
+        OpenStack API during provisioning.
     """
 
     def __init__(self, session=None, cloud_region=None, dry_run=False):
-        self._api = _os_api.API(session=session, cloud_region=cloud_region)
+        if cloud_region is None:
+            if session is None:
+                raise TypeError('Either session or cloud_region must '
+                                'be provided')
+            self.connection = connection.Connection(session=session)
+        elif session is not None:
+            raise TypeError('Either session or cloud_region must be provided, '
+                            'but not both')
+        else:
+            session = cloud_region.get_session()
+            self.connection = connection.Connection(config=cloud_region)
+
+        self._api = _os_api.API(session, self.connection)
         self._dry_run = dry_run
 
-    def reserve_node(self, resource_class=None, capabilities=None,
-                     candidates=None):
+    def reserve_node(self, resource_class=None, conductor_group=None,
+                     capabilities=None, traits=None, candidates=None,
+                     predicate=None):
         """Find and reserve a suitable node.
 
         Example::
@@ -60,11 +79,18 @@ class Provisioner(object):
 
         :param resource_class: Requested resource class. If ``None``, a node
             with any resource class can be chosen.
+        :param conductor_group: Conductor group to pick the nodes from.
+            Value ``None`` means any group, use empty string "" for nodes
+            from the default group.
         :param capabilities: Requested capabilities as a dict.
+        :param traits: Requested traits as a list of strings.
         :param candidates: List of nodes (UUIDs, names or `Node` objects)
             to pick from. The filters (for resource class and capabilities)
             are still applied to the provided list. The order in which
             the nodes are considered is retained.
+        :param predicate: Custom predicate to run on nodes. A callable that
+            accepts a node and returns ``True`` if it should be included,
+            ``False`` otherwise. Any exceptions are propagated to the caller.
         :return: reserved `Node` object.
         :raises: :py:class:`metalsmith.exceptions.ReservationFailed`
         """
@@ -72,30 +98,37 @@ class Provisioner(object):
 
         if candidates:
             nodes = [self._api.get_node(node) for node in candidates]
-            if resource_class:
-                nodes = [node for node in nodes
-                         if node.resource_class == resource_class]
+            filters = [
+                _scheduler.NodeTypeFilter(resource_class, conductor_group),
+            ]
         else:
-            nodes = self._api.list_nodes(resource_class=resource_class)
+            nodes = self._api.list_nodes(resource_class=resource_class,
+                                         conductor_group=conductor_group)
+            if not nodes:
+                raise exceptions.NodesNotFound(resource_class, conductor_group)
             # Ensure parallel executions don't try nodes in the same sequence
             random.shuffle(nodes)
+            # No need to filter by resource_class and conductor_group any more
+            filters = []
 
-        if not nodes:
-            raise exceptions.ResourceClassNotFound(resource_class,
-                                                   capabilities)
+        LOG.debug('Candidate nodes: %s', nodes)
 
-        LOG.debug('Ironic nodes: %s', nodes)
+        filters.append(_scheduler.CapabilitiesFilter(capabilities))
+        filters.append(_scheduler.TraitsFilter(traits))
+        if predicate is not None:
+            filters.append(_scheduler.CustomPredicateFilter(predicate))
 
-        filters = [_scheduler.CapabilitiesFilter(resource_class, capabilities),
-                   _scheduler.ValidationFilter(self._api,
-                                               resource_class, capabilities)]
-        reserver = _scheduler.IronicReserver(self._api, resource_class,
-                                             capabilities)
+        reserver = _scheduler.IronicReserver(self._api)
         node = _scheduler.schedule_node(nodes, filters, reserver,
                                         dry_run=self._dry_run)
+
+        update = {}
         if capabilities:
-            node = self._api.update_node(
-                node, {'/instance_info/capabilities': capabilities})
+            update['/instance_info/capabilities'] = capabilities
+        if traits:
+            update['/instance_info/traits'] = traits
+        if update:
+            node = self._api.update_node(node, update)
 
         LOG.debug('Reserved node: %s', node)
         return node
@@ -162,9 +195,10 @@ class Provisioner(object):
 
         return hostname
 
-    def provision_node(self, node, image, nics=None, root_disk_size=None,
-                       config=None, hostname=None, netboot=False,
-                       capabilities=None, wait=None):
+    def provision_node(self, node, image, nics=None, root_size_gb=None,
+                       swap_size_mb=None, config=None, hostname=None,
+                       netboot=False, capabilities=None, traits=None,
+                       wait=None, root_disk_size=None):
         """Provision the node with the given image.
 
         Example::
@@ -172,19 +206,22 @@ class Provisioner(object):
          provisioner.provision_node("compute-1", "centos",
                                     nics=[{"network": "private"},
                                           {"network": "external"}],
-                                    root_disk_size=50,
+                                    root_size_gb=50,
                                     wait=3600)
 
         :param node: Node object, UUID or name. Will be reserved first, if
             not reserved already. Must be in the "available" state with
             maintenance mode off.
-        :param image: Image name or UUID to provision.
+        :param image: Image source - one of :mod:`~metalsmith.sources`,
+            `Image` name or UUID.
         :param nics: List of virtual NICs to attach to physical ports.
             Each item is a dict with a key describing the type of the NIC:
             either a port (``{"port": "<port name or ID>"}``) or a network
             to create a port on (``{"network": "<network name or ID>"}``).
-        :param root_disk_size: The size of the root partition. By default
+        :param root_size_gb: The size of the root partition. By default
             the value of the local_gb property is used.
+        :param swap_size_mb: The size of the swap partition. It's an error
+            to specify it for a whole disk image.
         :param config: :py:class:`metalsmith.InstanceConfig` object with
             the configuration to pass to the instance.
         :param hostname: Hostname to assign to the instance. Defaults to the
@@ -194,8 +231,13 @@ class Provisioner(object):
             overwrites the capabilities set by :meth:`reserve_node`.
             Note that the capabilities are not checked against the ones
             provided by the node - use :meth:`reserve_node` for that.
+        :param traits: Requested traits of the node. If present, overwrites
+            the traits set by :meth:`reserve_node`. Note that the traits are
+            not checked against the ones provided by the node - use
+            :meth:`reserve_node` for that.
         :param wait: How many seconds to wait for the deployment to finish,
             None to return immediately.
+        :param root_disk_size: DEPRECATED, use ``root_size_gb``.
         :return: :py:class:`metalsmith.Instance` object with the current
             status of provisioning. If ``wait`` is not ``None``, provisioning
             is already finished.
@@ -203,22 +245,22 @@ class Provisioner(object):
         """
         if config is None:
             config = _config.InstanceConfig()
+        if isinstance(image, six.string_types):
+            image = sources.GlanceImage(image)
+        if root_disk_size is not None:
+            warnings.warn("root_disk_size is deprecated, use root_size_gb "
+                          "instead", DeprecationWarning)
+            root_size_gb = root_disk_size
+
         node = self._check_node_for_deploy(node)
         created_ports = []
         attached_ports = []
 
         try:
             hostname = self._check_hostname(node, hostname)
-            root_disk_size = _utils.get_root_disk(root_disk_size, node)
+            root_size_gb = _utils.get_root_disk(root_size_gb, node)
 
-            try:
-                image = self._api.get_image(image)
-            except Exception as exc:
-                raise exceptions.InvalidImage(
-                    'Cannot find image %(image)s: %(error)s' %
-                    {'image': image, 'error': exc})
-
-            LOG.debug('Image: %s', image)
+            image._validate(self.connection)
 
             nics = self._get_nics(nics or [])
 
@@ -235,17 +277,16 @@ class Provisioner(object):
 
             capabilities['boot_option'] = 'netboot' if netboot else 'local'
 
-            updates = {'/instance_info/image_source': image.id,
-                       '/instance_info/root_gb': root_disk_size,
+            updates = {'/instance_info/root_gb': root_size_gb,
                        '/instance_info/capabilities': capabilities,
                        '/extra/%s' % _CREATED_PORTS: created_ports,
                        '/extra/%s' % _ATTACHED_PORTS: attached_ports,
                        '/instance_info/%s' % _os_api.HOSTNAME_FIELD: hostname}
-
-            for prop in ('kernel', 'ramdisk'):
-                value = getattr(image, '%s_id' % prop, None)
-                if value:
-                    updates['/instance_info/%s' % prop] = value
+            updates.update(image._node_updates(self.connection))
+            if traits is not None:
+                updates['/instance_info/traits'] = traits
+            if swap_size_mb is not None:
+                updates['/instance_info/swap_mb'] = swap_size_mb
 
             LOG.debug('Updating node %(node)s with %(updates)s',
                       {'node': _utils.log_node(node), 'updates': updates})
@@ -294,7 +335,8 @@ class Provisioner(object):
             nic_type, nic_id = next(iter(nic.items()))
             if nic_type == 'network':
                 try:
-                    network = self._api.get_network(nic_id)
+                    network = self.connection.network.find_network(
+                        nic_id, ignore_missing=False)
                 except Exception as exc:
                     raise exceptions.InvalidNIC(
                         'Cannot find network %(net)s: %(error)s' %
@@ -303,7 +345,8 @@ class Provisioner(object):
                     result.append((nic_type, network))
             else:
                 try:
-                    port = self._api.get_port(nic_id)
+                    port = self.connection.network.find_port(
+                        nic_id, ignore_missing=False)
                 except Exception as exc:
                     raise exceptions.InvalidNIC(
                         'Cannot find port %(port)s: %(error)s' %
@@ -318,7 +361,7 @@ class Provisioner(object):
         """Create and attach ports on given networks."""
         for nic_type, nic in nics:
             if nic_type == 'network':
-                port = self._api.create_port(network_id=nic.id)
+                port = self.connection.network.create_port(network_id=nic.id)
                 created_ports.append(port.id)
                 LOG.info('Created port %(port)s for node %(node)s on '
                          'network %(net)s',
@@ -354,7 +397,8 @@ class Provisioner(object):
         for port_id in created_ports:
             LOG.debug('Deleting port %s', port_id)
             try:
-                self._api.delete_port(port_id)
+                self.connection.network.delete_port(port_id,
+                                                    ignore_missing=False)
             except Exception as exc:
                 LOG.warning('Failed to delete neutron port %(port)s: %(exc)s',
                             {'port': port_id, 'exc': exc})
@@ -506,3 +550,14 @@ class Provisioner(object):
                     self._api.get_node(inst, accept_hostname=True))
                 for inst in instances
             ]
+
+    def list_instances(self):
+        """List instances deployed by metalsmith.
+
+        :return: list of :py:class:`metalsmith.Instance` objects.
+        """
+        nodes = self._api.list_nodes(provision_state=None, associated=True)
+        instances = [i for i in
+                     (_instance.Instance(self._api, node) for node in nodes)
+                     if i._is_deployed_by_metalsmith]
+        return instances
