@@ -114,15 +114,37 @@ def schedule_node(nodes, filters, reserver, dry_run=False):
     assert False, "BUG: %s.fail did not raise" % reserver.__class__.__name__
 
 
+class NodeTypeFilter(Filter):
+    """Filter that checks resource class and conductor group."""
+
+    def __init__(self, resource_class=None, conductor_group=None):
+        self.resource_class = resource_class
+        self.conductor_group = conductor_group
+
+    def __call__(self, node):
+        return (
+            (self.resource_class is None or
+             node.resource_class == self.resource_class) and
+            (self.conductor_group is None or
+             node.conductor_group == self.conductor_group)
+        )
+
+    def fail(self):
+        raise exceptions.NodesNotFound(self.resource_class,
+                                       self.conductor_group)
+
+
 class CapabilitiesFilter(Filter):
     """Filter that checks capabilities."""
 
-    def __init__(self, resource_class, capabilities):
-        self._resource_class = resource_class
+    def __init__(self, capabilities):
         self._capabilities = capabilities
         self._counter = collections.Counter()
 
     def __call__(self, node):
+        if not self._capabilities:
+            return True
+
         try:
             caps = _utils.get_capabilities(node)
         except Exception:
@@ -159,67 +181,84 @@ class CapabilitiesFilter(Filter):
         message = ("No available nodes found with capabilities %(req)s, "
                    "existing capabilities: %(exist)s" %
                    {'req': requested, 'exist': existing or 'none'})
-        raise exceptions.CapabilitiesNotFound(message,
-                                              self._resource_class,
-                                              self._capabilities)
+        raise exceptions.CapabilitiesNotFound(message, self._capabilities)
 
 
-class ValidationFilter(Filter):
-    """Filter that runs validation on nodes."""
+class TraitsFilter(Filter):
+    """Filter that checks traits."""
 
-    def __init__(self, api, resource_class, capabilities):
-        self._api = api
-        # These are only used for better exceptions
-        self._resource_class = resource_class
-        self._capabilities = capabilities
-        self._failed_validation = []
+    def __init__(self, traits):
+        self._traits = traits
+        self._counter = collections.Counter()
 
     def __call__(self, node):
-        try:
-            self._api.validate_node(node.uuid)
-        except RuntimeError as exc:
-            message = ('Node %(node)s failed validation: %(err)s' %
-                       {'node': _utils.log_node(node), 'err': exc})
-            LOG.warning(message)
-            self._failed_validation.append(message)
+        if not self._traits:
+            return True
+
+        traits = node.traits or []
+        LOG.debug('Traits for node %(node)s: %(traits)s',
+                  {'node': _utils.log_node(node), 'traits': traits})
+        for trait in traits:
+            self._counter[trait] += 1
+
+        missing = set(self._traits) - set(traits)
+        if missing:
+            LOG.debug('Node %(node)s does not have traits %(missing)s',
+                      {'node': _utils.log_node(node), 'missing': missing})
             return False
 
         return True
 
     def fail(self):
-        errors = ", ".join(self._failed_validation)
-        message = "All available nodes have failed validation: %s" % errors
-        raise exceptions.ValidationFailed(message,
-                                          self._resource_class,
-                                          self._capabilities)
+        existing = ", ".join("%s (%d node(s))" % item
+                             for item in self._counter.items())
+        requested = ', '.join(self._traits)
+        message = ("No available nodes found with traits %(req)s, "
+                   "existing traits: %(exist)s" %
+                   {'req': requested, 'exist': existing or 'none'})
+        raise exceptions.TraitsNotFound(message, self._traits)
+
+
+class CustomPredicateFilter(Filter):
+
+    def __init__(self, predicate):
+        self.predicate = predicate
+        self._failed_nodes = []
+
+    def __call__(self, node):
+        if not self.predicate(node):
+            self._failed_nodes.append(node)
+            return False
+
+        return True
+
+    def fail(self):
+        message = 'No nodes satisfied the custom predicate %s' % self.predicate
+        raise exceptions.CustomPredicateFailed(message, self._failed_nodes)
 
 
 class IronicReserver(Reserver):
 
-    def __init__(self, api, resource_class, capabilities):
+    def __init__(self, api):
         self._api = api
-        # These are only used for better exceptions
-        self._resource_class = resource_class
-        self._capabilities = capabilities
+        self._failed_nodes = []
+
+    def validate(self, node):
+        try:
+            self._api.validate_node(node)
+        except RuntimeError as exc:
+            message = ('Node %(node)s failed validation: %(err)s' %
+                       {'node': _utils.log_node(node), 'err': exc})
+            LOG.warning(message)
+            raise exceptions.ValidationFailed(message)
 
     def __call__(self, node):
-        result = self._api.reserve_node(node, instance_uuid=node.uuid)
-
-        # Try validation again to be sure nothing has changed
-        validator = ValidationFilter(self._api, self._resource_class,
-                                     self._capabilities)
-        if not validator(result):
-            LOG.warning('Validation of node %s failed after reservation',
-                        _utils.log_node(node))
-            try:
-                self._api.release_node(node)
-            except Exception:
-                LOG.exception('Failed to release the reserved node %s',
-                              _utils.log_node(node))
-            validator.fail()
-
-        return result
+        try:
+            self.validate(node)
+            return self._api.reserve_node(node, instance_uuid=node.uuid)
+        except Exception:
+            self._failed_nodes.append(node)
+            raise
 
     def fail(self):
-        raise exceptions.AllNodesReserved(self._resource_class,
-                                          self._capabilities)
+        raise exceptions.NoNodesReserved(self._failed_nodes)
